@@ -9,14 +9,16 @@ from nginx_analysis.dataclasses import (
     RootNginxConfig,
     filter_unique,
     get_children_recursive,
+    get_parents_recursive,
 )
 
 
 def nginx_to_regex(nginx: str) -> str:
     """
-    Convert a nginx-style regex to a python regex
+    Convert a nginx-style regex to a python regex.
+    Wildcard files are converted to regexes that match any file in the directory.
     """
-    return nginx.replace("*", ".*")
+    return nginx.replace("*", r"[^/]*")
 
 
 def set_parents_in_include(root_config: RootNginxConfig, block_config: NginxLineConfig):
@@ -27,6 +29,7 @@ def set_parents_in_include(root_config: RootNginxConfig, block_config: NginxLine
                 for nested_line_config in nested_file_config.parsed:
                     nested_line_config.parent = block_config
                     block_config.children.append(nested_line_config)
+                    set_parents_in_include(root_config, nested_line_config)
 
 
 def set_parents_of_blocks(root_config: RootNginxConfig, line_config: NginxLineConfig):
@@ -104,44 +107,62 @@ def is_partial_direct_match(
     return None
 
 
-def find_matches_in_children(
+def get_matching_lines_in_children(
     line: NginxLineConfig, filters: List[DirectiveFilter]
 ) -> Tuple[List[NginxLineConfig], List[DirectiveFilter]]:
     """
-    Check if the line matches filters. If it matches, return the line,
-    the line's neighbours and all children. If it doesn't match, check
-    if any of the children match the filters. If they do, return all
-    descendants of the line that match the filters, including the current
-    line.
+    Check if the line matches filters. If it matches, return the line.
+    If it doesn't match, check if any of the children match the filters.
     We also return a list of filters that matched, so we can check if
     all filters were matched eventually.
     If there are no matches, return an empty list for both the matching
     lines and the matched filters.
     """
+    if not filters:
+        # No filters, meaning that the line matches directly
+        return [line], []
+
+    logger.debug(f"Checking if line {line} matches filters: {filters}")
     matched_filter = is_partial_direct_match(line, filters)
     if matched_filter:
         logger.debug(f"Found match in children: {line}")
         all_matched_filters = set([matched_filter])
         # Search for remaining filters in children, as the parent
         # might still be looking for other filters
-        remaining_filters = [f for f in filters if f != matched_filter]
         for child in line.children:
-            _, child_matched_filters = find_matches_in_children(
-                child, remaining_filters
-            )
-            all_matched_filters.update(child_matched_filters)
-        return [line, *line.neighbours, *get_children_recursive(line)], list(
-            all_matched_filters
-        )
+            remaining_filters = [f for f in filters if f not in all_matched_filters]
+            if remaining_filters:
+                logger.debug(
+                    f"Looking for remaining filters in child {child}: {remaining_filters}"
+                )
+                _, child_matched_filters = get_matching_lines_in_children(
+                    child, remaining_filters
+                )
+                all_matched_filters.update(child_matched_filters)
+            else:
+                logger.debug(
+                    f"Found all filters in children of line {line}. We should stop!"
+                )
+                break
+        return [line], list(all_matched_filters)
 
     # All filters must match at least one child
     matched_filters = set()
-    matches = [line]
+    matches = []
     for child in line.children:
-        child_matches, child_matched_filters = find_matches_in_children(child, filters)
-        if child_matches:
-            matches.extend(child_matches)
-            matched_filters.update(child_matched_filters)
+        remaining_filters = [f for f in filters if f not in matched_filters]
+        if remaining_filters:
+            child_matches, child_matched_filters = get_matching_lines_in_children(
+                child, remaining_filters
+            )
+            if child_matches:
+                matches.extend(child_matches)
+                matched_filters.update(child_matched_filters)
+        else:
+            logger.debug(
+                f"Found all filters in children of line {line}. We should stop!"
+            )
+            break
 
     if len(matched_filters) == len(filters):
         logger.debug(
@@ -151,6 +172,25 @@ def find_matches_in_children(
     return [], []
 
 
+def expand_upon_direct_match(
+    match: NginxLineConfig, matched_filters: List[DirectiveFilter]
+) -> List[NginxLineConfig]:
+    """
+    Expand upon matching lines by recursively looking through
+    parents and neighbours. However, we don't want to include any
+    lines that have the same directive as in the filters.
+    F.e., if the filter looks for location=/banaan, we don't want
+    to include other recursive location lines.
+    """
+    matching_lines = []
+    for neighbour in match.neighbours:
+        matching_lines.append(neighbour)
+        matching_lines.extend(get_children_recursive(neighbour))
+    parents = get_parents_recursive(match)
+    matching_lines.extend(parents)
+    return matching_lines
+
+
 def filter_config(
     lines: Iterator[NginxLineConfig],
     filters: List[DirectiveFilter],
@@ -160,10 +200,10 @@ def filter_config(
     """
     matching_lines = []
     for line in lines:
-        child_matches, matched_filters = find_matches_in_children(line, filters)
-        if len(matched_filters) == len(filters):
-            logger.debug(f"Matched all ({len(matched_filters)}) filters: {line}")
-            matching_lines.extend(child_matches)
+        child_matches, _ = get_matching_lines_in_children(line, filters)
+        matching_lines.extend(child_matches)
+        for match in child_matches:
+            matching_lines.extend(expand_upon_direct_match(match, filters))
 
     return filter_unique(matching_lines)
 
