@@ -66,9 +66,12 @@ class DirectiveFilter(BaseModel):
     def __hash__(self) -> int:
         return hash(str(self))
 
-    def __str__(self) -> str:
+    def __repr__(self) -> str:
         return f"{self.directive} -> {self.value}"
 
+    def __str__(self) -> str:
+        return self.__repr__()
+
 
 class NginxLineConfig(BaseModel):
     directive: str
@@ -103,9 +106,12 @@ class NginxLineConfig(BaseModel):
     def __hash__(self) -> int:
         return hash(str(self))
 
-    def __str__(self) -> str:
+    def __repr__(self) -> str:
         return f"{self.file}:{self.line} -> {self.directive}"
 
+    def __str__(self) -> str:
+        return self.__repr__()
+
 
 class NginxFileConfig(BaseModel):
     file: Path
@@ -121,9 +127,12 @@ class NginxFileConfig(BaseModel):
         comparison_fields = ["file", "status", "errors", "parsed"]
         return compare_objects(self, other, comparison_fields)
 
-    def __str__(self) -> str:
+    def __repr__(self) -> str:
         return f"{self.file}"
 
+    def __str__(self) -> str:
+        return self.__repr__()
+
     @property
     def lines(self) -> Generator[NginxLineConfig, None, None]:
         for line in self.parsed:
@@ -183,3 +192,9 @@ class RootNginxConfig(BaseModel):
 
         comparison_fields = ["status", "errors", "config"]
         return compare_objects(self, other, comparison_fields)
+
+    def __repr__(self) -> str:
+        return f"{self.status} {self.root_file}"
+
+    def __str__(self) -> str:
+        return self.__repr__()
